;-------------------------------------------------------------------------------
;	Periferie DS1 - alarm
;	DS adresa 00000011
;
;	16F84A / 4MHz			
;	vse OFF, osc. XT, bez preruseni
;
;	02/2016 Milan rajer
;-------------------------------------------------------------------------------
;
;                             --------\/--------
;                       BUS  -|RA2          RA1|-  PGY
;                            -|RA3          RA0|-  PGX
;                            -|RA4         OSC1|-  \ Krystal 4,00 Mhz
;                            -|MCLR        OSC2|-  /
;                        0V  -|Vss          Vdd|-  +5V
;                    rele 1  -|RB0          RB7|-  ICSP
;                    rele 2  -|RB1          RB6|-  ICSP
;                    rele 3  -|RB2          RB5|-  rele 6
;                    rele 4  -|RB3          RB4|-  rele 5
;                             ------------------
;
;-------------------------------------------------------------------------------

	LIST P=16F84, R=DEC
	INCLUDE <P16F84.INC>

;-------------------------------
;	Nastaveni pojistek
;-------------------------------

	org	0x2007		; adresa nastaveni pojistek pro PIC16F84
	dw _XT_OSC & _PWRTE_OFF & _WDT_OFF & _CP_OFF

;-------------------------------
;	Nastaveni dat v EEPROM
;-------------------------------

	org	0x2100		;tabulka v pameti EEPROM (64 bytu)
;		----------------------------------------------
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
;		----------------------------------------------

;--------------------------------------------------------------------------------
;	Nastaveni parametru
;-------------------------------
W	equ	0
F	equ	1
;--------------------------------------------------------------------------------
;	Definice registru, bitu a pinu
;-------------------------------
SYS	equ	0x0C
;-----------------------------------
;|   |   |   |   |   |   |   | TMP |
;-----------------------------------
#define	TMP	SYS,0
;       ------------------------
TMP1	equ	0x0D		;pomocny registr
TMP2	equ	0x0E		;pomocny registr
TMP3	equ	0x0F		;pomocny registr
TMP4	equ	0x10		;pomocny registr
TMP5	equ	0x11		;pomocny registr
;       ------------------------
CEK1	equ	0x12		;casova smycka
CEK2	equ	0x13		;casova smycka
CEK3	equ	0x14		;casova smycka
;       ------------------------
DAT	equ	0x15		;datovy registr
;
ALARM	equ	0x16		;registr jednotlivych alarmu
AL_CIT	equ	0x17		;registr alarmu
PGSTAV	equ	0x18		;registr vystupu alarmu
;-------------------------------
;	Sbernice
;-------------------------------
#define	BUS	PORTA,2		;sbernice
#define	BUS_T	TRISA,2		;tris registr sbernice
;-------------------------------
;	Periferie
;-------------------------------
#define	RELE1	PORTB,0		;rele 1
#define	RELE2	PORTB,1		;rele 2
#define	RELE3	PORTB,2		;rele 3
#define	RELE4	PORTB,3		;rele 4
#define	RELE5	PORTB,4		;rele 5
#define	RELE6	PORTB,5		;rele 6
;
#define	PGX	PORTA,0		;PGX vystup alarmu
#define	PGY	PORTA,1		;PGY vystup alarmu
;--------------------------------------------------------------------------------
;--------------------------------------------------------------------------------
;	Zacatek behu programu
;-------------------------------
	org	0x0000
	goto	initial
;--------------------------------------------------------------------------------
;	Knihovny
;-------------------------------
	#include	<piclib_bus_4MHz.inc>		;komunikace po sbernici
;--------------------------------------------------------------------------------
;--------------------------------------------------------------------------------
;	Inicializace po zapnuti
;-------------------------------
;-------------------------------
;	nastaveni portu
;-------------------------------
initial	bcf	STATUS,RP0	;banka 0
	bcf	STATUS,RP1
;-------------------------------
	clrf	PORTA		;vymazani vsech portu
	clrf	PORTB
;-------------------------------
	bsf	STATUS,RP0	;banka 1
	bcf	STATUS,RP1
;-------------------------------
	movlw	b'11111'
	movwf	TRISA		;port A vstupni/vystupni
	movlw	b'11000000'
	movwf	TRISB		;port B vstupni/vystupni
;       ------------------------
	bcf	STATUS,RP0	;banka 0
	bcf	STATUS,RP1
;--------------------------------------------------------------------------------
;--------------------------------------------------------------------------------
;
;	*****	*****	*****	****	*****
;	*	  *	*   *	*   *	  *
;	*****	  *	***** 	****	  *
;	    *	  *	*   *	*  *	  *
;	*****	  *	*   *	*   *	  *
;
;--------------------------------------------------------------------------------
;--------------------------------------------------------------------------------
start	clrf	ALARM		;vynuluj registry
	clrf	PGSTAV
;-------------------------------
;	Zjisteni stavu vystupu alarmu
;-------------------------------
mereni	btfss	PGX		;stav PGX
	bcf	PGSTAV,0
	btfsc	PGX
	bsf	PGSTAV,0
;       ------------------------
	btfss	PGY		;stav PGY
	bcf	PGSTAV,1
	btfsc	PGY
	bsf	PGSTAV,1
;-------------------------------
;	Neustale probihajici detekce reset impulsu
;-------------------------------
rutina	btfsc	BUS		;cekej na log 0
	goto	rutina
;-------------------------------
	movlw	.30		;test delky impulsu log 0 po dobu 120us
        movwf	CEK1
;       ------------------------
	btfsc	BUS		;pokud je impuls kratsi, vrat se na rutinu
	goto	rutina
;       ------------------------
	decfsz	CEK1,1
        goto	$-3		;..jinak pokracuj
;-------------------------------
	movlw	.3		;cekej max 1200us na konec reset impulsu (log 1)
        movwf	CEK2
rut12   movlw	.100
        movwf	CEK1
;       ------------------------
rut11	btfsc	BUS		;pokud behem testu prejde BUS do log 1..
	goto	nac_adr		;..pokracuj v prijmu
;       ------------------------
	decfsz	CEK1,1
        goto	rut11
        decfsz	CEK2,1
        goto	rut12		;pokud se log 1 neobevi, vrat se na rutinu
	goto	rutina
;-------------------------------
;	Prijem adresy periferie (00000011)
;-------------------------------
nac_adr	call	bus_dat		;precte byte ze sbernice
;       ------------------------
	movlw	b'00000011'	;je tazano toto zarizeni?
	subwf	DAT,W
	btfss	STATUS,Z	;pokud ne, vrat se na rutinu, jinak pokracuj
	goto	rutina
;-------------------------------
;	Prijem dat ze sbernice
;-------------------------------
prijmi	call	bus_dat		;precte byte ze sbernice
	movf	DAT,W
	movwf	ALARM
;-------------------------------
;	Vyslani dat na sbernici
;-------------------------------
vysli	call	bus_rw		;prodleva pred zacatkem vysilani
;       ------------------------
	movf	PGSTAV,W	;T1
	call	w_bus
;       ------------------------
	movlw	b'01010101'	;kontrolni byte
	call	w_bus
;--------------------------------------------------------------------------------
;	ALARM (bude vyvolan po 4. cteni ze sbernice)
;--------------------------------------------------------------------------------
	movf	ALARM,F		;je nejaky alarm?
	btfss	STATUS,Z	;..pokud ano..
	goto	jealarm		;..osetri alarm..
	clrf	AL_CIT		;..jinak vynuluj citac alarmu
	bcf	RELE1		;..a zrus vsechny alarmy
	bcf	RELE2
	bcf	RELE3
	bcf	RELE4
	bcf	RELE5
	bcf	RELE6
;       ------------------------
	goto	mereni		;..a vrat se na mereni
;-------------------------------
jealarm	btfsc	AL_CIT,2	;doslo uz k 4. cteni alarmu?
	goto	je8alar		;.. pak spust alarm
;       ------------------------
	incf	AL_CIT,F	;..jinak inkrementuj citac alarmu
;       ------------------------
	goto	mereni		;..a vrat se na mereni
;-------------------------------
je8alar	btfsc	ALARM,0
	goto	rel1
	btfsc	ALARM,1
	goto	rel2
	btfsc	ALARM,2
	goto	rel3
	btfsc	ALARM,3
	goto	rel4
	btfsc	ALARM,4
	goto	rel5
	btfsc	ALARM,5
	goto	rel6
	goto	mereni
;       ------------------------
rel1	bsf	RELE1		;aktivuj alarm 1
	bcf	RELE2
	bcf	RELE3
	bcf	RELE4
	bcf	RELE5
	bcf	RELE6
;       ------------------------
	goto	mereni		;a vrat se na mereni
;       ------------------------
rel2	bcf	RELE1		;aktivuj alarm 2
	bsf	RELE2
	bcf	RELE3
	bcf	RELE4
	bcf	RELE5
	bcf	RELE6
;       ------------------------
	goto	mereni		;a vrat se na mereni
;       ------------------------
rel3	bcf	RELE1		;aktivuj alarm 3
	bcf	RELE2
	bsf	RELE3
	bcf	RELE4
	bcf	RELE5
	bcf	RELE6
;       ------------------------
	goto	mereni		;a vrat se na mereni
;       ------------------------
rel4	bcf	RELE1		;aktivuj alarm 4
	bcf	RELE2
	bcf	RELE3
	bsf	RELE4
	bcf	RELE5
	bcf	RELE6
;       ------------------------
	goto	mereni		;a vrat se na mereni
;       ------------------------
rel5	bcf	RELE1		;aktivuj alarm 5
	bcf	RELE2
	bcf	RELE3
	bcf	RELE4
	bsf	RELE5
	bcf	RELE6
;       ------------------------
	goto	mereni		;a vrat se na mereni
;       ------------------------
rel6	bcf	RELE1		;aktivuj alarm 6
	bcf	RELE2
	bcf	RELE3
	bcf	RELE4
	bcf	RELE5
	bsf	RELE6
;       ------------------------
	goto	mereni		;a vrat se na mereni
;--------------------------------------------------------------------------------
	end